package org.processmining.activityclusterarraycreator.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Dimension;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifier;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifierManager;
import org.processmining.activityclusterarraycreator.parameters.ModifyActivityClusterArrayParameters;
import org.processmining.framework.util.ui.widgets.ProMList;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ModifyActivityClusterArrayDialog extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8779850269383145104L;

	public ModifyActivityClusterArrayDialog(ActivityClusterArray clusters,
			final ModifyActivityClusterArrayParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL, TableLayoutConstants.FILL, 30, 30, 30, 30, 30 } };
		setLayout(new TableLayout(size));
		setOpaque(false);
		
		DefaultListModel<String> listModel = new DefaultListModel<String>();
		for (ActivityClusterArrayModifier modifier: ActivityClusterArrayModifierManager.getInstance().getModifiers()) {
			listModel.addElement(modifier.getName());
		}
		final ProMList<String> list = new ProMList<String>("Select miner", listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		final String defaultModifier = parameters.getModifier();
		list.setSelection(defaultModifier);
		list.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				List<String> selected = list.getSelectedValuesList();
				if (selected.size() == 1) {
					parameters.setModifier(selected.get(0));
				} else {
					/*
					 * Nothing selected. Revert to selection of default classifier.
					 */
					list.setSelection(defaultModifier);
					parameters.setModifier(defaultModifier);
				}
			}
		});
		list.setPreferredSize(new Dimension(100, 100));
		add(list, "0, 0");

		final NiceSlider clusterSlider = SlickerFactory.instance().createNiceIntegerSlider("Number of Clusters", 0, clusters.getClusters().size(),
				parameters.getNrOfClusters(), Orientation.HORIZONTAL);
		clusterSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setNrOfClusters(clusterSlider.getSlider().getValue());
			}
		});
		add(clusterSlider, "0, 1");

		final NiceSlider cohesionSlider = SlickerFactory.instance().createNiceIntegerSlider("Cohesion Weight", 0, 100,
				parameters.getCohesionWeight(), Orientation.HORIZONTAL);
		cohesionSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setCohesionWeight(cohesionSlider.getSlider().getValue());
			}
		});
		add(cohesionSlider, "0, 2");

		final NiceSlider couplingSlider = SlickerFactory.instance().createNiceIntegerSlider("Coupling Weight", 0, 100,
				parameters.getCouplingWeight(), Orientation.HORIZONTAL);
		couplingSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setCouplingWeight(couplingSlider.getSlider().getValue());
			}
		});
		add(couplingSlider, "0, 3");

		final NiceSlider balanceSlider = SlickerFactory.instance().createNiceIntegerSlider("Balance Weight", 0, 100,
				parameters.getBalanceWeight(), Orientation.HORIZONTAL);
		balanceSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setBalanceWeight(balanceSlider.getSlider().getValue());
			}
		});
		add(balanceSlider, "0, 4");

		final NiceSlider overlapSlider = SlickerFactory.instance().createNiceIntegerSlider("Overlap Weight", 0, 100,
				parameters.getOverlapWeight(), Orientation.HORIZONTAL);
		overlapSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setOverlapWeight(balanceSlider.getSlider().getValue());
			}
		});
		add(overlapSlider, "0, 5");

	}

}
