package org.processmining.activityclusterarraycreator.metrics.impl;

import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.metrics.ActivityClusterArrayMetric;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public class ActivityClusterArrayCohesionMetric implements ActivityClusterArrayMetric {

	private double weight;
	
	public ActivityClusterArrayCohesionMetric(double weight) {
		this.setWeight(weight);
	}
	
	public double compute(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {
		double cohesion = 0.0;
		
		for (Set<XEventClass> cluster: clusters.getClusters()) {
			
			double clusterCohesion = 0.0;
			for(XEventClass firstActivity: cluster){
				for(XEventClass secondActivity: cluster){
					clusterCohesion += graph.getCausality(firstActivity, secondActivity);
				}
			}
			
			cohesion += clusterCohesion / (cluster.size() * cluster.size());
		}

		return cohesion / clusters.getClusters().size();
	}

	public double getWeight() {
		return weight;
	}

	public void setWeight(double weight) {
		this.weight = weight;
	}

}
