package org.processmining.activityclusterarraycreator.modifiers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayBruteForceModifier;
import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayOverlapProximityModifier;
import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayOverlapScoringModifier;
import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayProximityModifier;
import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayRandomModifier;
import org.processmining.activityclusterarraycreator.modifiers.impl.ActivityClusterArrayScoringModifier;

public class ActivityClusterArrayModifierManager {
	private static ActivityClusterArrayModifierManager instance = null;
	private List<ActivityClusterArrayModifier> modifiers;
	private ActivityClusterArrayModifier defaultModifier;
	private boolean sorted;

	private ActivityClusterArrayModifierManager() {
		modifiers = new ArrayList<ActivityClusterArrayModifier>();
		defaultModifier = null;
		sorted = false;
		register(new ActivityClusterArrayBruteForceModifier(), false);
		register(new ActivityClusterArrayRandomModifier(), false);
		register(new ActivityClusterArrayProximityModifier(), false);
		register(new ActivityClusterArrayOverlapProximityModifier(), true);
		register(new ActivityClusterArrayScoringModifier(), false);
		register(new ActivityClusterArrayOverlapScoringModifier(), false);
	}

	public static ActivityClusterArrayModifierManager getInstance() {
		if (instance == null) {
			instance = new ActivityClusterArrayModifierManager();
		}
		return instance;
	}
	
	public void register(ActivityClusterArrayModifier modifier, boolean isDefault) {
		modifiers.add(modifier);
		if (isDefault) {
			defaultModifier = modifier;
		}
		sorted = false;
	}

	public List<ActivityClusterArrayModifier> getModifiers() {
		if (!sorted) {
			Collections.sort(modifiers, new Comparator<ActivityClusterArrayModifier>() {
				public int compare(ActivityClusterArrayModifier m1, ActivityClusterArrayModifier m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return modifiers;
	}
	
	public boolean isDefault(ActivityClusterArrayModifier miner) {
		return miner == defaultModifier;
	}
	
	public ActivityClusterArrayModifier getModifier(String name) {
		if (name != null) {
			for (ActivityClusterArrayModifier modifier : modifiers) {
				if (name.equals(modifier.getName())) {
					return modifier;
				}
			}
		}
		return defaultModifier;
	}

}
