package org.processmining.activityclusterarraycreator.modifiers.impl;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarray.models.impl.ActivityClusterArrayFactory;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifier;
import org.processmining.activityclusterarraycreator.modifiers.ActivityClusterArrayModifierParameters;
import org.processmining.activityclusterarraycreator.utils.ClusterUtils;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.framework.plugin.PluginContext;

public abstract class ActivityClusterArrayAbstractScoringModifier implements ActivityClusterArrayModifier {

	public ActivityClusterArray apply(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph,
			ActivityClusterArrayModifierParameters parameters) {
//		ActivityClusterArrayMetric weighted = new ActivityClusterArrayWeightedMetric(
//				new ActivityClusterArrayCohesionMetric(parameters.getCohesionWeight()),
//				new ActivityClusterArrayCouplingMetric(parameters.getCouplingWeight()),
//				new ActivityClusterArrayBalanceMetric(parameters.getBalanceWeight()));

		ActivityClusterArray oldClusters = ActivityClusterArrayFactory.createActivityClusterArray();
		oldClusters.init(clusters);
		for (Set<XEventClass> cluster : clusters.getClusters()) {
			oldClusters.addCluster(cluster);
			oldClusters.setInputs(cluster, clusters.getInputs(cluster));
			oldClusters.setOutputs(cluster, clusters.getOutputs(cluster));
		}

		while (parameters.getNrOfClusters() < oldClusters.getClusters().size()) {
			System.out.println("Clusters left: " + oldClusters.getClusters().size());
			double bestScore = 0.0;
			ActivityClusterArray bestClusters = null;

			for (int i = 0; i < oldClusters.getClusters().size(); i++) {
				Set<XEventClass> firstCluster = oldClusters.getClusters().get(i);
				for (int j = i + 1; j < oldClusters.getClusters().size(); j++) {
					Set<XEventClass> secondCluster = oldClusters.getClusters().get(j);
					boolean check;
					check = preMergeCheck(firstCluster, secondCluster);
					if (check) {
						ActivityClusterArray newClusters;
						Set<Set<XEventClass>> mergeClusters = new HashSet<Set<XEventClass>>();
						mergeClusters.add(firstCluster);
						mergeClusters.add(secondCluster);
						newClusters = ClusterUtils.merge(oldClusters, mergeClusters);
						if (newClusters.isConsistent()) {
							double score;
							score = parameters.getScore(context, newClusters, graph);
//							score = weighted.compute(context, newClusters, graph);
							if (score > bestScore) {
								bestScore = score;
								bestClusters = newClusters;
							}
						}
					}
				}
			}
			if (bestClusters == null) {
				return oldClusters;
			}
			oldClusters = bestClusters;
		}

		return oldClusters;
	}

	public abstract boolean preMergeCheck(Set<XEventClass> firstCluster, Set<XEventClass> secondCluster);
}
