package org.processmining.activityclusterarraycreator.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.algorithms.DiscoverClustersFromEventLogAlgorithm;
import org.processmining.activityclusterarraycreator.connections.DiscoverClustersFromEventLogConnection;
import org.processmining.activityclusterarraycreator.dialogs.DiscoverClustersFromEventLogDialog;
import org.processmining.activityclusterarraycreator.help.DiscoverClustersFromEventLogHelp;
import org.processmining.activityclusterarraycreator.parameters.DiscoverClustersFromEventLogParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Discover Clusters", parameterLabels = { "Event Log", "Parameters" }, returnLabels = { "Activity Cluster Array" }, returnTypes = { ActivityClusterArray.class }, help = DiscoverClustersFromEventLogHelp.TEXT)
public class DiscoverClustersFromEventLogPlugin extends DiscoverClustersFromEventLogAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Clusters, UI", requiredParameterLabels = { 0 })
	public ActivityClusterArray runUI(UIPluginContext context, XLog log) {
		DiscoverClustersFromEventLogParameters parameters = new DiscoverClustersFromEventLogParameters(log);
		DiscoverClustersFromEventLogDialog dialog = new DiscoverClustersFromEventLogDialog(log, parameters);
		InteractionResult result = context.showWizard("Configure discovery (classifier, miner)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, log, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public ActivityClusterArray mineUI(UIPluginContext context, XLog log) {
		return runUI(context, log);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Clusters, Default", requiredParameterLabels = { 0 })
	public ActivityClusterArray runDefault(PluginContext context, XLog log) {
		DiscoverClustersFromEventLogParameters parameters = new DiscoverClustersFromEventLogParameters(log);
		return runConnection(context, log, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public ActivityClusterArray discoverDefault(PluginContext context, XLog log) {
		return runDefault(context, log);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Clusters, Default", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray run(PluginContext context, XLog log, DiscoverClustersFromEventLogParameters parameters) {
		return runConnection(context, log, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public ActivityClusterArray discoverDefault(PluginContext context, XLog log,
			DiscoverClustersFromEventLogParameters parameters) {
		return run(context, log, parameters);
	}

	private ActivityClusterArray runConnection(PluginContext context, XLog log,
			DiscoverClustersFromEventLogParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<DiscoverClustersFromEventLogConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						DiscoverClustersFromEventLogConnection.class, context, log);
				for (DiscoverClustersFromEventLogConnection connection : connections) {
					if (connection.getObjectWithRole(DiscoverClustersFromEventLogConnection.LOG).equals(log)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(DiscoverClustersFromEventLogConnection.CLUSTERS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		ActivityClusterArray clusters = apply(context, log, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new DiscoverClustersFromEventLogConnection(log, clusters, parameters));
		}
		return clusters;
	}
}
