package org.processmining.activityclusterarraycreator.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.activityclusterarraycreator.algorithms.ModifyActivityClusterArrayAlgorithm;
import org.processmining.activityclusterarraycreator.connections.ModifyActivityClusterArrayConnection;
import org.processmining.activityclusterarraycreator.dialogs.ModifyActivityClusterArrayDialog;
import org.processmining.activityclusterarraycreator.help.ModifyActivityClusterArrayHelp;
import org.processmining.activityclusterarraycreator.parameters.ModifyActivityClusterArrayParameters;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Modify Clusters", parameterLabels = { "Activity Cluster Array", "Causal Activity Graph", "Parameters" }, returnLabels = { "Modified Activity Cluster Array" }, returnTypes = { ActivityClusterArray.class }, userAccessible = true, help = ModifyActivityClusterArrayHelp.TEXT)
public class ModifyActivityClusterArrayPlugin extends ModifyActivityClusterArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Modify Clusters, UI", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray runUI(UIPluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph) {
		ModifyActivityClusterArrayParameters parameters = new ModifyActivityClusterArrayParameters(clusters);
		ModifyActivityClusterArrayDialog dialog = new ModifyActivityClusterArrayDialog(clusters, parameters);
		InteractionResult result = context.showWizard("Configure modifier (modifier, parameters)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, clusters, graph, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public ActivityClusterArray modifyUI(UIPluginContext context, ActivityClusterArray clusters,
			CausalActivityGraph graph) {
		return runUI(context, clusters, graph);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Modify Clusters, Default", requiredParameterLabels = { 0, 1 })
	public ActivityClusterArray runDefault(PluginContext context, ActivityClusterArray clusters,
			CausalActivityGraph graph) {
		ModifyActivityClusterArrayParameters parameters = new ModifyActivityClusterArrayParameters(clusters);
		return runConnection(context, clusters, graph, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public ActivityClusterArray modifyDefault(PluginContext context, ActivityClusterArray clusters,
			CausalActivityGraph graph) {
		return runDefault(context, clusters, graph);
	}

	@PluginVariant(variantLabel = "Modify Clusters, Parameters", requiredParameterLabels = { 0, 1, 2 })
	public ActivityClusterArray run(PluginContext context, ActivityClusterArray clusters, CausalActivityGraph graph,
			ModifyActivityClusterArrayParameters parameters) {
		return runConnection(context, clusters, graph, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public ActivityClusterArray modifyParameters(PluginContext context, ActivityClusterArray clusters,
			CausalActivityGraph graph, ModifyActivityClusterArrayParameters parameters) {
		return run(context, clusters, graph, parameters);
	}

	private ActivityClusterArray runConnection(PluginContext context, ActivityClusterArray clusters,
			CausalActivityGraph graph, ModifyActivityClusterArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ModifyActivityClusterArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(ModifyActivityClusterArrayConnection.class,
						context, clusters, graph);
				for (ModifyActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(ModifyActivityClusterArrayConnection.CLUSTERS).equals(clusters)
							&& connection.getObjectWithRole(ModifyActivityClusterArrayConnection.GRAPH).equals(graph)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(ModifyActivityClusterArrayConnection.MODIFIEDCLUSTERS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		ActivityClusterArray modifiedClusters = apply(context, clusters, graph, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ModifyActivityClusterArrayConnection(clusters, graph, modifiedClusters, parameters));
		}
		return modifiedClusters;
	}
}
